/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2006 Dr. Lars H. Hahn

*/

package haushalt.auswertung.planung;

import haushalt.daten.Euro;

import javax.swing.table.AbstractTableModel;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.1/2005.02.03
 */

/*
 * 2006.02.03 Erste Version
 */

public class PlanungTableModel extends AbstractTableModel {
  private static final long serialVersionUID = 1L;

  private static final String[] spaltenNamen =
    {"Kategorie","Betrag","verwenden"};
  private final Planung planung;
  
  public PlanungTableModel(Planung planung) {
    this.planung = planung;
  }

  public int getRowCount() {
    return planung.getAnzahlKategorien()+1;
  }

  public int getColumnCount() {
    return 3;
  }

  public String getColumnName(int col) {
    return spaltenNamen[col];
  }

  public Class<?> getColumnClass(int col) {
    switch(col) {
      case 0 : return String.class;
      case 1 : return Euro.class; 
      case 2 : return Boolean.class;
      default: return null;
    }
  }

  public boolean isCellEditable(int row, int col) {
    if((col == 0) || 
       ((col == 1) && !planung.kategorieVerwenden(row).booleanValue()) ||
       (row == planung.getAnzahlKategorien()))
      return false;
    return true;
  }

  public Object getValueAt(int row, int col) {
    if(row == planung.getAnzahlKategorien()) {
      switch(col) {
      case 0: return "SUMME";
      case 1: return planung.getSumme();
      case 2: return new Boolean(false);
      }
    }
    else {
      switch(col) {
      case 0: return planung.getKategorie(row);
      case 1: if(planung.kategorieVerwenden(row))
        return planung.getBetrag(row);
      else
        return new Euro();
      case 2: return planung.kategorieVerwenden(row);
      }
    }
    return null;
  }
  
  public void setValueAt(Object value, int row, int col) {
    switch(col) {
    case 1: planung.setBetrag(row, new Euro(""+value)); break;
    case 2: planung.setVerwenden(row, (Boolean)value); break;
    }
    this.fireTableCellUpdated(planung.getAnzahlKategorien(), 1);
  }

}
