/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2006 Dr. Lars H. Hahn

*/

package haushalt.daten;

import haushalt.daten.zeitraum.AbstractZeitraum;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

/**
 * Reprsentiert ein Buchungsdatum.
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */

public class Datum implements Comparable<Datum>, Cloneable {

  private static final DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, Locale.GERMAN);
  private Date wert;

  public Datum() {
    wert = heute();
  }

  public Datum(int tag, int monat, int jahr) {
    GregorianCalendar cal = new GregorianCalendar(jahr, monat-1, tag);
    wert = cal.getTime();
  }

  public Datum(String wert) {
    String heute = df.format(new Date());
    if(wert.length() < 8)
      wert = wert + heute.substring(wert.length());
    try {
      this.wert = df.parse(wert);
    }
    catch(java.text.ParseException e) {
      this.wert = heute();
    }
  }
  
  private Date heute() {
    String heute = df.format(new Date());
    try {
      return df.parse(heute);
    }
    catch(java.text.ParseException e) {
      e.printStackTrace();
      return null;
    }
  }
  
	public String toString() {
		return df.format(wert);
	}

  public long getTime() {
    return wert.getTime();
  }

  public Datum addiereTage(int tage) {
    Datum datum = new Datum();
    datum.wert = new Date(getTime()+tage*86400000L);
    return datum;
  }

  public int getTag() {
    GregorianCalendar gc = new GregorianCalendar();
    gc.setTime(wert);
    return gc.get(Calendar.DAY_OF_MONTH);
  }

  public int getMonat() {
    GregorianCalendar gc = new GregorianCalendar();
    gc.setTime(wert);
    return gc.get(Calendar.MONTH)+1;
  }

  public int getJahr() {
    GregorianCalendar gc = new GregorianCalendar();
    gc.setTime(wert);
    return gc.get(Calendar.YEAR);
  }

  public boolean istImZeitraum(AbstractZeitraum zeitraum) {
    return ((compareTo(zeitraum.getStartDatum()) >= 0) &&
            (compareTo(zeitraum.getEndDatum()) < 0));
  }

	// -- E/A-Funktionen -------------------------------------------------------

	public void laden(DataInputStream in) throws IOException {
		try {
			wert = df.parse(in.readUTF());
		}
		catch(java.text.ParseException e) {
			wert = heute();
			throw new IOException();
		}
	}

	public void speichern(DataOutputStream out) throws IOException {
		out.writeUTF(toString());
	}
	
	// -- Methoden fuer Interface: Cloneable --------------------

	public Object clone() {
		return new Datum(getTag(), getMonat(), getJahr());
	}
	
	// -- Methoden fuer Interface: Comparable -------------------

	public int compareTo(Datum datum) {
    return wert.compareTo(datum.wert);
  }

}