/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2006 Dr. Lars H. Hahn

*/

package haushalt.gui;

import java.awt.event.KeyEvent;

import haushalt.daten.Datum;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.1/2006.01.28
 */

 /*
  * 2006.01.28 Plus/Minus-Tasten zum Verndern des Datums
  *            hinzugefgt
  */

public class DatumField extends DeleteableTextField {

  private static final long serialVersionUID = 1L;

  public DatumField() {
    this(new Datum());
  }

  public DatumField(Datum datum) {
    super(""+datum, 8); // 8 Spalten
  }

  protected Document createDefaultModel() {
    return new DatumDocument();
  }

  public void keyReleased(KeyEvent e) {
    super.keyReleased(e);
    if(e.getKeyCode() == KeyEvent.VK_PLUS) {
      Datum datum = new Datum(getText());
      setText(""+datum.addiereTage(1));
    } else if(e.getKeyCode() == KeyEvent.VK_MINUS) {
      Datum datum = new Datum(getText());
      setText(""+datum.addiereTage(-1));
    }
  }

  private class DatumDocument extends PlainDocument {
    private static final long serialVersionUID = 1L;
    public void insertString(int offs, String str, AttributeSet a)
      throws BadLocationException {
      char[] source = str.toCharArray();
      char[] result = new char[8]; // max. 8 Zeichen: tt.mm.yy
      int j = 0;
      int anzChar = (source.length < 8)?source.length:8;

      for (int i = 0; i < anzChar; i++) {
        if (((source[i] == '.') || (source[i] == '-')) && ((offs+i == 2) || (offs+i == 5))) {
          result[j++] = '.';
        }
        else {
          if (Character.isDigit(source[i])) {
            if((offs+i == 2) || (offs+i == 5)) {
              result[j++] = '.';
            }
            result[j++] = source[i];
          }
        }
      }
      if(j+getLength() > 8)
        j = 8 - getLength();
      super.insertString(offs, new String(result, 0, j), a);
    }
  }
}