/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2006 Dr. Lars H. Hahn

*/

package haushalt.gui.dialoge;

import haushalt.daten.Datenbasis;
import haushalt.daten.Euro;
import haushalt.daten.EinzelKategorie;
import haushalt.gui.DatumField;
import haushalt.gui.DeleteableTextField;
import haushalt.gui.EuroField;
import haushalt.gui.EuroRenderer;
import haushalt.gui.Haushalt;
import haushalt.gui.KategorieCellEditor;
import haushalt.gui.KategorieRenderer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumnModel;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */

public class DlgAutoBuchung extends JDialog {
  private static final long serialVersionUID = 1L;

  // GUI-Komponenten
  private final JPanel buttonPane = new JPanel();
  private final JButton buttonOK = new JButton("OK");
  private final JButton buttonDelete;
  protected final AutoBuchungTableModel tableModel;
  protected final JTable table;

  public DlgAutoBuchung(final Haushalt haushalt, final Datenbasis db) {
    super(haushalt.getFrame(), "Automatische Buchungen", true);
		tableModel = new AutoBuchungTableModel(db);
    table = new JTable(tableModel);
		table.setSurrendersFocusOnKeystroke(true);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setSelectionBackground(haushalt.getFarbeSelektion());
		table.setGridColor(haushalt.getFarbeGitter());
    table.setPreferredScrollableViewportSize(new Dimension(500,200));

    // Action erzeugen
    Action action = new AbstractAction("Lschen", haushalt.bildLaden("Delete16.gif")) {
      private static final long serialVersionUID = 1L;
  		public void actionPerformed(ActionEvent e) {
  		  if(e.getActionCommand().equals("Lschen")) {
	        TableCellEditor cellEditor = table.getCellEditor();
	        if(cellEditor != null)
	          cellEditor.cancelCellEditing();
	        int row = table.getSelectedRow();
	        if(row == -1) {
	          JOptionPane.showMessageDialog(haushalt.getFrame(),
	            "Keine Zeile selektiert!",
	            "Alt-D: Automatische Buchung",
	            JOptionPane.WARNING_MESSAGE);
	        }
	        else if(row == tableModel.getRowCount()-1) {
	          JOptionPane.showMessageDialog(haushalt.getFrame(),
	            "Die letzte (Eingabe-)Zeile kann nicht gelscht werden.",
	            "Alt-D: Automatische Buchung",
	            JOptionPane.WARNING_MESSAGE);
	        }
	        else {
	          tableModel.entferneZeile(row);
	        }
  		  }
  		}      
    };
		action.putValue(Action.SHORT_DESCRIPTION, "Lscht eine Auto-Buchung");
		KeyStroke key = KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK);
		action.putValue(Action.ACCELERATOR_KEY, key);
		action.putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_D));
    
		// Cell-Editoren erzeugen
    TableColumnModel columnModel = table.getColumnModel();
    DatumField datumField = new DatumField();
    columnModel.getColumn(0).setCellEditor(new DefaultCellEditor(datumField));
    DeleteableTextField textField = new DeleteableTextField();
    columnModel.getColumn(1).setCellEditor(new DefaultCellEditor(textField));
    columnModel.getColumn(2).setCellEditor(new KategorieCellEditor(haushalt, db));
    EuroField euroField = new EuroField();
    columnModel.getColumn(3).setCellEditor(new DefaultCellEditor(euroField));
    JComboBox comboBox1 = new JComboBox(db.getRegisterNamen());
    columnModel.getColumn(4).setCellEditor(new DefaultCellEditor(comboBox1));
    JComboBox comboBox2 = new JComboBox(db.getAutoBuchungIntervallNamen());
    columnModel.getColumn(5).setCellEditor(new DefaultCellEditor(comboBox2));
		
    // Cell-Renderer erzeugen
    table.setDefaultRenderer(EinzelKategorie.class, new KategorieRenderer());
    table.setDefaultRenderer(Euro.class, new EuroRenderer());

    Container contentPane = getContentPane();
    contentPane.add(new JScrollPane(table), BorderLayout.CENTER);
    contentPane.add(buttonPane, BorderLayout.SOUTH);
    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
    		int anzahl = db.ausfuehrenAutoBuchungen();
    		if(anzahl > 0)
          JOptionPane.showMessageDialog(haushalt.getFrame(),
              "Es wurden "+anzahl+" automatische Buchung ausgefhrt.");
      }
    });
		buttonDelete = new JButton(action);
    buttonPane.add(buttonDelete);
    buttonPane.add(buttonOK);
  }

  public void zeigeDialog() {
    pack();
    setVisible(true);
  }

}